import 'package:logger/logger.dart';
import 'package:oauth2/oauth2.dart';
import 'package:suga_core/src/storage/storage.dart';
import 'package:synchronized/extension.dart';

class Oauth2Manager {
  final Uri _endpoint;
  final String _identifier;
  final String _secret;
  final bool _basicAuth;
  final Storage<Credentials> _credentialStorage;
  final Logger? _logger;

  Oauth2Manager({
    required Uri endpoint,
    required String identifier,
    required String secret,
    bool basicAuth = false,
    required Storage<Credentials> credentialStorage,
    Logger? logger,
  })  : _endpoint = endpoint,
        _identifier = identifier,
        _secret = secret,
        _basicAuth = basicAuth,
        _credentialStorage = credentialStorage,
        _logger = logger;

  Future<Credentials> login({
    required String username,
    required String password,
  }) async {
    _logger?.d("Logging in - username $username, password $password");

    final Client client = await resourceOwnerPasswordGrant(
      _endpoint,
      username,
      password,
      identifier: _identifier,
      secret: _secret,
      basicAuth: _basicAuth,
    );

    final Credentials credentials = client.credentials;
    _credentialStorage.set(credentials);
    _logger?.d("Logged in - credentials ${credentials.toJson()}");
    return credentials;
  }

  Future<Credentials> refresh({Credentials? credentials}) async {
    credentials ??= _credentialStorage.get();

    _logger?.d("Refreshing - credentials ${credentials?.toJson()}");

    if (credentials == null) {
      throw StateError("Credentials must be not null");
    }

    final newCredentials = await credentials.refresh(
      identifier: _identifier,
      secret: _secret,
      basicAuth: _basicAuth,
    );
    _credentialStorage.set(newCredentials);

    _logger?.d("Refreshed - credentials ${newCredentials.toJson()}");
    return newCredentials;
  }

  Future<bool> isLoggedIn() async {
    final credentials = _credentialStorage.get();
    return credentials != null && !credentials.isExpired;
  }

  Future<String?> getAccessToken({
    bool refreshIfExpired = true,
  }) async {
    var credentials = _credentialStorage.get();

    if (credentials == null) {
      return null;
    }

    if (refreshIfExpired && credentials.isExpired) {
      await synchronized(() async {
        if (credentials != null && credentials!.isExpired) {
          credentials = await refresh(credentials: credentials);
        }
      });
    }

    return credentials?.accessToken;
  }
}
